function [ number ] = dtw( Cx, Cref, Nref)
% word recognition via dynamic time warping (DTW) of matrices with cepstral coeffs 

[ Nx, Nc ] = size(Cx);       % number of: cepstral vectors, cepstral coeffs

for nref = 1 : length(Nref)  % compare Cx with Cref of all reference patterns
 %  Calculate distance d(ns, nr) between signal (ns) and ref (nr) cepstra
    Nr = Nref( nref );                         % number of vectors with ref cepstra
    Q = round( 0.2 * max(Nx,Nr) );             % coefficient of path width
    d = Inf*ones(Nx,Nr);                       % init. of distance matrix
    tg=(Nr-Q)/(Nx-Q);                          % init. of angle tangent
    for nx = 1:Nx                              % for each cepstrum of the word
        down(nx) = max( 1, floor(tg*nx-Q*tg)); % lower limit
        up(nx)   = min( Nr, ceil(tg*nx+Q));    % upper limit
      % for nr = 1 : Nr
        for nr = down(nx) : up(nx)             % for each cepstrum of ref word
            d(nx,nr)=sqrt( sum( (Cx(nx, 1:Nc) - Cref(nr, 1:Nc, nref)).^2 )); % distance
        end
    end
 %  Calculation of accumulated (accum) distance g()
    g = d;                                             % initialization
    for nx = 2:Nx, g(nx,1) = g(nx-1,1) + d(nx,1); end  % accum of 1st column
    for nr = 2:Nr, g(1,nr) = g(1,nr-1) + d(1,nr); end  % accum of first row
    for nx = 2:Nx                             % accum vertically (word)
        for nr = max( down(nx), 2 ) : up(nx)  % accum horizontally (ref)
            dd = d(nx,nr);                    % distance: word "nx", ref "nr"
            temp(1) = g(nx-1,nr) + dd;        % go up
            temp(2) = g(nx-1,nr-1) + 2*dd;    % go over diagonal (right up)
            temp(3) = g(nx,nr-1) + dd;        % go right
            g(nx,nr) = min( temp );           % choose accumulated value
        end
    end
    glob(nref) = g(Nx,Nr)/sqrt(Nx^2+Nr^2)     % accum value of the "shortest" path
end
[ xxx number ] = min( glob );                 % number of ref with the lowest acc value